import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import React from "react";
import Slide from "@mui/material/Slide";
import Snackbar from "@mui/material/Snackbar";
import { Icon, Stack, Typography } from "@mui/material";
import Box from "@mui/material/Box";
import { useTranslation } from "react-i18next";
import nerdUII18n from "../../i18n";
import { Alert, Button, IconButton, LinearProgress } from "@mui/joy";
import { useDispatch } from "react-redux";
import { setOpenFeedbackDialog } from "../../redux/nerdFeedback";
import { useSelector } from "react-redux";
import { nerdNotifierNotificationSelector } from "../../redux";
import { severityIcon } from "../const";
import CloseRoundedIcon from "@mui/icons-material/CloseRounded";
import * as Sentry from "@sentry/react";
let direction = undefined;
/**
 *
 * @param {NerdNotificationProps & openHookAction} param0 components props
 * @returns {JSX.Element} NonIntrusive nerdNotifier
 */
export function NonIntrusive({ handleClose, open, hideReportButton, }) {
    const { title, transition = "up", severity = "primary", subtitle, body, position = {
        vertical: "bottom",
        horizontal: "right",
    }, autoHideDuration = 5000, autohide, error, } = useSelector(nerdNotifierNotificationSelector) || {};
    direction = transition;
    const [progress, setProgress] = React.useState(0);
    const { t } = useTranslation("common", { i18n: nerdUII18n });
    const timer = React.useRef(0);
    const dispatch = useDispatch();
    React.useEffect(() => {
        timer.current = window.setInterval(() => {
            setProgress((prevProgress) => {
                return prevProgress >= 100
                    ? 0
                    : prevProgress + 100 / (autoHideDuration / 10);
            });
        }, 10);
        return () => {
            clearInterval(timer.current);
        };
    }, []);
    React.useEffect(() => {
        if (progress >= 100 && autohide) {
            clearInterval(timer.current);
            handleClose();
        }
    }, [progress, autohide]);
    return (_jsx(Box, { children: _jsx(Snackbar, { open: open, autoHideDuration: autohide ? autoHideDuration : null, TransitionComponent: Transition, anchorOrigin: position, children: _jsxs(Stack, { spacing: 1, children: [autohide && (_jsx(LinearProgress, { determinate: true, value: progress, color: severity })), _jsx(Alert, { variant: "solid", color: severity, sx: {
                            width: "100%",
                            alignItems: "center",
                        }, startDecorator: severityIcon[severity], endDecorator: _jsx(IconButton, { onClick: handleClose, sx: {
                                ml: 1,
                            }, variant: "solid", size: "sm", color: severity, children: _jsx(CloseRoundedIcon, {}) }), children: _jsxs("div", { children: [title && (_jsx(Typography, { color: "inherit", variant: "body1", children: title })), subtitle && (_jsx(Typography, { color: "inherit", variant: "body2", children: subtitle })), body && (_jsx(Typography, { color: "inherit", variant: "caption", children: body }))] }) }), Sentry.getCurrentHub().getClient()?.getDsn()?.publicKey &&
                        (severity === "danger" || severity === "warning") &&
                        !hideReportButton ? (_jsx(Button, { size: "sm", onClick: () => {
                            dispatch(setOpenFeedbackDialog({ open: true, error }));
                            handleClose();
                        }, startDecorator: _jsx(Icon, { fontSize: "small", className: "fa-regular fa-comment-exclamation" }), color: "neutral", children: t("components:nerdNotifier.report_issue") })) : null] }) }) }));
}
/**
 *
 * @param {TransitionProps} props Component props
 * @returns {JSX.Element} Transition Component
 */
function Transition(props) {
    return _jsx(Slide, { ...props, direction: direction });
}
